package com.starcite.portrange;

import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.Executors;

import org.jboss.netty.bootstrap.ClientBootstrap;
import org.jboss.netty.channel.ChannelFuture;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.ChannelPipelineFactory;
import org.jboss.netty.channel.Channels;
import org.jboss.netty.channel.socket.nio.NioClientSocketChannelFactory;


/**
 * Created by IntelliJ IDEA.
 * User: andy.song
 * Date: 12/28/10
 * Time: 2:43 PM
 * To change this template use File | Settings | File Templates.
 */
public class EchoClient {

    public static void main(String[] args) throws Exception {
        // Print usage if no argument is specified.
        if (args.length < 2 || args.length > 3) {
            System.err.println(
                    "Usage: " + EchoClient.class.getSimpleName() +
                            " <host> <port> [<first message size>]");
            return;
        }

        // Parse options.
        final String host = args[0];
        final int port = Integer.parseInt(args[1]);
        final int firstMessageSize;
        if (args.length == 3) {
            firstMessageSize = Integer.parseInt(args[2]);
        } else {
            firstMessageSize = 256;
        }

        // Configure the client.
        EchoClient client = new EchoClient();
        for (int i = 0; i < 2; ++i) {
            client.new ClientThread(host, port, firstMessageSize).start();
        }

    }

    public class ClientThread extends Thread {
        private String host;
        private int port;
        private int firstMessageSize;

        public ClientThread(String host, int port, int messageSize) {
            this.host = host;
            this.port = port;
            this.firstMessageSize = messageSize;
        }

        public void run() {
            ClientBootstrap bootstrap = new ClientBootstrap(
                    new NioClientSocketChannelFactory(
                            Executors.newCachedThreadPool(),
                            Executors.newCachedThreadPool()));

            // Set up the pipeline factory.
            bootstrap.setPipelineFactory(new ChannelPipelineFactory() {
                public ChannelPipeline getPipeline() throws Exception {
                    return Channels.pipeline(
                            new EchoClientHandler(firstMessageSize));
                }
            });

            // Start the connection attempt.

            Collection<ChannelFuture> queueFutures = new ArrayList<ChannelFuture>();
            for (int i = 0; i < 1000; ++i) {
                ChannelFuture future = bootstrap.connect(new InetSocketAddress(host, port));
                queueFutures.add(future);
            }


            // Wait until the connection is closed or the connection attempt fails.
            for (ChannelFuture future : queueFutures) {
                future.getChannel().getCloseFuture().awaitUninterruptibly();
            }

            // Shut down thread pools to exit.
            bootstrap.releaseExternalResources();
        }
    }


}
